//********************************************************************
//*** ResourcePool/SQL/Bind.hh
//*** Copyright (c) 2003-2004 by Markus Winand <mws@fatalmind.com>
//*** $Id: Bind.hh,v 1.3 2004/11/17 14:39:15 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_SQL_BIND_HH
#define RESOURCEPOOL_SQL_BIND_HH

#ifndef INCLUDED_GC_PTR_HH
#include "mm/gc_ptr.hh"
#endif

#include <vector>

namespace fatalmind {
namespace SQL {

template<class VBC, template <class> class CT>
class Bind {
	public:
		/**
		 * Binds an actual input value for an placeholder in the 
		 * SQL statement.
		 *
		 * All binding is done by value! this means the passed data
		 * are copied to internal storage. chaning your original
		 * copy of your data after passing it to bind() is therefore
		 * safe.
		 * 
		 * \param pos The position indicator. The number of 
		 *        placeholder (?) in the SQL string countet from the 
		 *        left starting at 0.
		 * \param val the actual value.
		 * \param nul NULL inidcator, if set to yes the actual value 
		 *        to be bound will be NULL. Please note that you still
		 *        need to provide a valid value!
		 */
		template<typename T>
		void bind(const int pos, const T& val, bool nul = false);
	protected:
		typedef std::vector<gc_ptr<VBC> > _argst;
		_argst _args;

	private:
		void instantiations();
};

} // namespace SQL
} // namespace fatalmind
#endif
