//********************************************************************
//*** ResourcePool/SQL/SQLException.hh
//*** Copyright (c) 2003-2004 by Markus Winand <mws@fatalmind.com>
//*** $Id: SQLException.hh,v 1.4 2004/11/17 14:39:15 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_SQL_SQLException_HH
#define RESOURCEPOOL_SQL_SQLException_HH

#ifndef INCLUDED_EXCEPTIONS_HH
#include "Exceptions/Exception.hh"
#endif

namespace fatalmind {
namespace SQL {

/**
 * General exception to be thrown by SQL commands.
 */
class SQLException: virtual public Exception {
	public:
		SQLException(const std::string& msg
			, const std::string& sqlstate // confirming to X/Open
			, const std::string& sqlstm   // the SQL statement which caused the exception
			, const bool         failover // Should this exception cause an failover in ResourcPool?
		):Exception(msg)
		, _sqlstate(sqlstate)
		, _sqlstm(sqlstm)
		, _failover(failover)
		{
		}

		const std::string& getSQL() const throw() {
			return _sqlstm;
		}

		bool isFailover() const throw() {
			return _failover;
		}

		const std::string& getSQLState() const throw() {
			return _sqlstate;
		}
		
		bool isSQLState(const char* const c) const {
			return !_sqlstate.compare(c);
		}

		const std::string getSQLStateClass() const {
			return _sqlstate.substr(0,2);
		}

		bool isSQLStateClass(const std::string& c) const {
			return !_sqlstate.compare(0, 2, c, 0, 2);
		}
			
		bool isSQLStateClass(const char* const c) const {
			return !_sqlstate.compare(0, 2, c, 0, 2);
		}

	private:
		const std::string _sqlstate;
		const std::string _sqlstm;
		const bool        _failover;
};

} // namespace SQL
} // namespace fatalmind

#endif
