//********************************************************************
//*** ResourcePool/SQL/SQLSelect.t.cc
//*** Copyright (c) 2003-2004 by Markus Winand <mws@fatalmind.com>
//*** $Id: SQLSelect.t.cc,v 1.11 2004/11/17 14:39:15 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */


#include "t/main.hh"

using namespace std;
using namespace fatalmind;

std::auto_ptr<RP> _rp;

void test_init()
{
	if (_rp.get() == 0) {
		_rp.reset(new RP(getFactory()));
	}
}

class SQLSelect_test: public RP::Command::SQLSelect {
	public:
		SQLSelect_test(const string& sql, int limit)
        : RP::Command::SQLSelect(sql)
		, _rows(0)
		, _limit(limit)
        {
        };
		SQLSelect_test* clone() const {
			return new SQLSelect_test(*this);
		}
		bool process() {
			return ++_rows <= _limit;
		}
        int _rows;
		int _limit;
};


void test_SQLSelect_init()
{
	test_init();
	init(*_rp);

	RP::Command::SQLExecute init("insert into hirsch (id, ukn) values (?, ?)");
	init.bind<std::string>(1, "hohoho ' kasdfjs \" sdf");
	for (int i = 0; i < 600; ++i) {
		init.bind(0, i);
		_rp->execute(init);
	}
}


void test_SQLSelect_one()
{
	string sql("select id, ukn from hirsch order by id");
	SQLSelect_test cmd(sql, 1000);
	int col1;
	std::string col2;

	cmd.bindout(0, col1);
	cmd.bindout(1, col2);

	_rp->execute(cmd);

	assertEqualT(int, col1, 599);
	assertEqualT(std::string, col2, "hohoho ' kasdfjs \" sdf");
}

void test_perf_SQLSelect_all()
{
	string sql("select id, ukn from hirsch");
	SQLSelect_test cmd(sql, 1000);
	int col1;
	std::string col2;

	cmd.bindout(0, col1);
	cmd.bindout(1, col2);

	for (int i = 0; i < 50; ++i) {
		_rp->execute(cmd);
	}

	assertEqualT(std::string, col2, "hohoho ' kasdfjs \" sdf");
}

void test_perf_SQLSelect_ten()
{
	string sql("select id, ukn from hirsch");
	SQLSelect_test cmd(sql, 10);
	int col1;
	std::string col2;

	cmd.bindout(0, col1);
	cmd.bindout(1, col2);

	for (int i = 0; i < 50; ++i) {
		_rp->execute(cmd);
	}

	assertEqualT(std::string, col2, "hohoho ' kasdfjs \" sdf");
}

void test_perf_SQLSelect_one()
{
	string sql("select id, ukn from hirsch where rownum <= 1");
	SQLSelect_test cmd(sql, 1);
	int col1;
	std::string col2;

	cmd.bindout(0, col1);
	cmd.bindout(1, col2);

	for (int i = 0; i < 50; ++i) {
		_rp->execute(cmd);
	}

	assertEqualT(std::string, col2, "hohoho ' kasdfjs \" sdf");
}

// TODO: test which does not match any record

void test_SQLSelect_cleanup() {
	drop(*_rp);
}
