//********************************************************************
//*** ResourcePool/oracle/Resource.hh
//*** Copyright (c) 2003-2004 by Markus Winand <mws@fatalmind.com>
//*** $Id: Resource.hh,v 1.19 2004/11/17 14:39:17 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_ORACLE_RESOURCE_HH
#define RESOURCEPOOL_ORACLE_RESOURCE_HH

#ifndef RESOURCEPOOL_CREATEEXCEPTION_HH
#include "ResourcePool/CreateException.hh"
#endif

#include "ResourceFwd.hh"

#ifndef RESOURCEPOOL_ORACLE_ORACLEEXCEPTION_HH
#include "OracleException.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_OCIHANDLE_HH
#include "OCIHandle.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_OCIENVHANDLEFWD_HH
#include "OCIEnvHandleFwd.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_STATEMENTCACHE_HH
#include "StatementCache.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_SELECTLISTMETADATA_HH
#include "SelectListMetaData.hh"
#endif

#ifndef OCI_ORACLE
#include <oci.h>
#endif

#include <string>

namespace fatalmind {

class oracleHandles {
	public:
		oracleHandles(const oracle::OCIEnvHandle& env
			, const oracle::OCIErrorHandle&   err
			, const oracle::OCIServerHandle&  srv
			, const oracle::OCISvcCtxHandle&  svc
			, const oracle::OCISessionHandle& ses
		)
		: _env(env) 
		, _err(err)
		, _srv(srv)
		, _svc(svc)
		, _ses(ses)
		{
		}

		virtual ~oracleHandles() {
		}

		OCIEnv* getEnv() const {
			return _env.get();
		}

		OCIError* getError() const {
			return _err.get();
		}

		OCIServer* getServer() const {
			return _srv.get();
		}

		OCISvcCtx* getSvcCtx() const {
			return _svc.get();
		}

		OCISession* getSession() const {
			return _ses.get();
		}

		//virtual oracle::SelectListMetaDataCacheInterface& getSelectListMetaDataCache() =0;
//		//typedef oracle::SelectListMetaDataCacheInterface::value_t slmdc_value_t;

	private:
		const oracle::OCIEnvHandle&     _env;
		const oracle::OCIErrorHandle&   _err;
		const oracle::OCIServerHandle&  _srv;
		const oracle::OCISvcCtxHandle&  _svc;
		const oracle::OCISessionHandle& _ses;
	// hidden stuff
		oracleHandles(const oracleHandles&);
		oracleHandles& operator=(const oracleHandles&);
};

template<class TM = DefaultThreadedModel >
class oracleHandlesWithCache: public oracleHandles {
	public:
		oracleHandlesWithCache(const oracle::OCIEnvHandle& env
			, const oracle::OCIErrorHandle&   err
			, const oracle::OCIServerHandle&  srv
			, const oracle::OCISvcCtxHandle&  svc
			, const oracle::OCISessionHandle& ses
			, gc_ptr<typename oracle::SelectListMetaDataCacheInterface<TM>::type > slmdc
		):oracleHandles(env, err, srv, svc, ses)
		, _slmdc(slmdc)
		{
		}
		virtual ~oracleHandlesWithCache()
		{
		}

		virtual typename oracle::SelectListMetaDataCacheInterface<TM>::type& getSelectListMetaDataCache() {
			return *_slmdc;
		}

//		typedef typename SLMDC::value_t slmdc_value_t;
	private:
		gc_ptr<typename oracle::SelectListMetaDataCacheInterface<TM>::type, TM> _slmdc;
	// hidden stuff
		oracleHandlesWithCache(const oracleHandlesWithCache<TM>&);
		oracleHandlesWithCache& operator=(const oracleHandlesWithCache<TM>&);
};


template<class TM>
class oracleResource {
    public:
		typedef typename oracle::SelectListMetaDataCacheInterface<TM>::type _slmdci_t;
        oracleResource<TM>(const std::string& service_name
            , const std::string& user
            , const std::string& pass
			, const oracle::OCIEnvHandle& env
			, gc_ptr<_slmdci_t, TM> slmdc
		);
		~oracleResource();
		bool precheck();
		bool postcheck();
		void close();
		void fail_close();
		oracleHandlesWithCache<TM>& get_plain_resource();

    private:
		const oracle::OCIEnvHandle&    _env;
		const oracle::OCIErrorHandle   _err;
		const oracle::OCIServerHandle  _srv;
		const oracle::OCISvcCtxHandle  _svc;
		const oracle::OCISessionHandle _ses;
		oracleHandlesWithCache<TM>           _plain;
};

} // namespace fatalmind
#endif

