//********************************************************************
//*** ResourcePool/oracle/SelectListMetaData.hh
//*** Copyright (c) 2003-2004 by Markus Winand <mws@fatalmind.com>
//*** $Id: SelectListMetaData.hh,v 1.8 2004/11/17 14:39:17 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_ORACLE_SELECTLISTMETADATA_HH
#define RESOURCEPOOL_ORACLE_SELECTLISTMETADATA_HH

#ifndef OCI_ORACLE
#include <oci.h>
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLEREFTYPE_HH
#include "oracleRefType.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_SQLSELECTROW_HH
#include "SQLSelectRow.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLESTATEMENT_HH
#include "oracleStatement.hh"
#endif

#include <vector>
#include <string>
#include <ostream>

namespace fatalmind {
namespace oracle {

class ColumnMetaData {
	public:
		ColumnMetaData(const size_t offst); // for usag in std containers
		//ColumnMetaData(const ub2 datatype, const ub2 datasize);
		ColumnMetaData(const ColumnMetaData& rho);

		ub2 getType() const throw();
		ub2 getSize() const throw();
		size_t getOffset() const throw();
		bool bufferedDefine() const throw();
		bool piecewise() const throw();

	private:
		const ub2* getTypePtr() const throw();
		const ub2* getSizePtr() const throw();

		ub2 _datatype;
		ub2 _datasize;
		size_t _offst;
	friend
		class SelectListMetaData;
	friend
		std::ostream& operator<<(std::ostream& s, const ColumnMetaData& rho);	
};

std::ostream& operator<<(std::ostream& s, const ColumnMetaData& rho);	

class SelectListMetaData {
	public:
		SelectListMetaData(const OCIStatementHandle& sth
			, const oracleHandles& hand
			, const std::string& SQL
		);

		size_t getBufferSz() const throw();
		int getNumSelectListItems() const;
		const ColumnMetaData& getColumnMetaData(const int pos) const;
		const oracleStatement& getOraSQL() const;

	private:
		typedef std::vector<ColumnMetaData> selectlist_t;
		selectlist_t _selectlist;
		oracleStatement _oraSQL;
		size_t _bufsz;
	friend
		std::ostream& operator<<(std::ostream& s, const SelectListMetaData& rho);
};

std::ostream& operator<<(std::ostream& s, const SelectListMetaData& rho);

/*
 * this class will be used to provide OCI with the
 * indicator variable and the place to store the actual size
 * it's placement constructed into the memory managed by
 * SelectListBuffer
 * in fact it's just a pattern for the memory
 */
class DefineMetaData {
	public:
		DefineMetaData(const ub2 bufsz, oracleRefType& out) 
		: _bufsz(bufsz)
		, _out(out)
		{
		}

		ub4* getSizePtr() throw() {
			return &_size;
		}

		void* getDataPtr() throw() {
			return (this + 1);
		}

		void resetSize() throw() {
			_size = _bufsz;
		}

		void setBufferSize(const ub4 bufsz) throw() {
			_bufsz = bufsz;
		}
		
		ub4 getSize() const throw() {
			return _size;
		}	

		void done(ub4 len, void* buf) {
			_out.done(len, buf);
		}
		
	private:
		ub4 _size;
		ub2 _bufsz;
		oracleRefType& _out;
};

class SelectListBuffer {
	public:
		SelectListBuffer(const gc_ptr<SelectListMetaData> slmd, oracleBindOut& bindout);
		~SelectListBuffer();

		int getNumSelectListItems() const;
		DefineMetaData* getDefineMetaData(const int pos) const;
		ub4* getSizePtr(const int pos) const;
		void* getDataPtr(const int pos) const;
		bool bufferedDefine(const int pos) const;
		
	private:
		const gc_ptr<SelectListMetaData> _slmd;		
		char* _buf;
};


} // namespace oracle
} // namespace fatalmind
#endif
