//********************************************************************
//*** Thread/CondVar.hh
//*** Copyright (c) 2003-2004 by Markus Winand <mws@fatalmind.com>
//*** $Id: CondVar.hh,v 1.18 2004/11/17 14:39:22 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef INCLUDED_CONDVAR_H
#define INCLUDED_CONDVAR_H


#ifndef INCLUDED_CONDVARFWD_H
#include "CondVarFwd.hh"
#endif

#ifndef INCLUDED_POSIXMUTEX_H
#include "PosixMutex.hh"
#endif

#ifndef INCLUDED_ERRNO_H
#include "Exceptions/Errno.hh"
#endif

#ifndef POSIX_TIMEFWD_HH
#include "POSIX/timeFwd.hh"
#endif

#ifndef INCLUDED_THREAD_LOCK_HH
#include "Lock.hh"
#endif

// TODO: should not be public visible
#include <pthread.h>
#include <sys/time.h>


namespace fatalmind {

template<class MutexType>
class CondVar: public MutexType {
		typedef MutexType super;
		using super::d_mutex;
//		using super::stat_unlock;
//		using super::stat_lock;
	public:
		CondVar() : MutexType() {
			Errno::ThrowOnError(
				pthread_cond_init(&d_condvar, NULL)
			);	
		}

		~CondVar() {
			pthread_cond_destroy(&d_condvar);
		}

		// TODO: nicer support methods
		template<class ThreadingModell>
		bool wait(const struct POSIX::timespec<ThreadingModell>* timeout) {
			struct ::timeval now;
			gettimeofday(&now, NULL);

			POSIX::timespec<ThreadingModell> ts(now.tv_sec, now.tv_usec*1000);
			ts+= *timeout;
			return waitAbsolute(&ts);
				
		}
		// TODO:: threading, default impl should copy timespec (locked)
		// specialization could skip this
		template<class ThreadingModell>
		bool waitAbsolute(const struct POSIX::timespec<ThreadingModell>* timeout) {
			super::stat_unlock();
			try {
				Errno::ThrowOnError(
					pthread_cond_timedwait(&d_condvar, &d_mutex, timeout)
				);	
			} catch (Errno::ETIMEDOUT e) {
				super::stat_lock();
				return false;
			}
			super::stat_lock();
			return true;
		}

		void wait() {
			super::stat_unlock();
			Errno::ThrowOnError(
				pthread_cond_wait(&d_condvar, &d_mutex)
			);	
			super::stat_lock();
		}

		void signal() {
			Errno::ThrowOnError(
				pthread_cond_signal(&d_condvar)
			);
		}

		void broadcast() {
			Errno::ThrowOnError(
				pthread_cond_broadcast(&d_condvar)
			);
		}
	protected:
		pthread_cond_t		d_condvar;

	private:
		
		
};

} // namespace fatalmind

#endif
